/*
 * Copyright (C) 2017 Guido Berhoerster <guido+pwm@berhoerster.name>
 *
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including
 * without limitation the rights to use, copy, modify, merge, publish,
 * distribute, sublicense, and/or sell copies of the Software, and to
 * permit persons to whom the Software is furnished to do so, subject to
 * the following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 * IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
 * CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
 * TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
 * SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */

#include "compat.h"

#include <stdlib.h>
/* glibc and Solaris 11 */
#include <sys/random.h>

#include "rand.h"

#define	RANDOM_CHUNK	256U

void
rand_buf(void *buf, size_t buf_size)
{
	unsigned char	*p = buf;
	ssize_t		nread;
	size_t		nsize = buf_size;

	while (nsize > 0) {
		nread = getrandom(p, nsize, 0);
		if (nread < 0) {
			if (errno == EINTR) {
				continue;
			}
			err(1, "read");
		}
		nsize -= nread;
		p += nread;
	}
}

uint32_t
rand_random(void)
{
	uint32_t	r;

	getrandom(r, sizeof (uint32_t), 0);

	return (r);
}

/* random number between 0 and upper_bound - 1 without modulo bias */
uint32_t
rand_uniform(uint32_t upper_bound)
{
	uint32_t	r;
	/* (2^32 - upper_bound) % upper_bound */
	uint32_t	threshold = -upper_bound % upper_bound;

	do {
		r = rand_random();
	} while (r < threshold);

	return (r % upper_bound);
}
